import { CodegenGenerator, CodegenConfig, CodegenGeneratorContext } from '@openapi-generator-plus/types';
/** Returns the string converted to a string that is safe as an identifier in java-like languages */
export declare function identifierSafe(value: string): string;
/**
 * Camel case and capitalize suitable for a class name. Doesn't change existing
 * capitalization in the value.
 * e.g. "FAQSection" remains "FAQSection", and "faqSection" will become "FaqSection"
 * @param value string to be turned into a class name
 */
export declare function classCamelCase(value: string): string;
export declare function identifierCamelCase(value: string): string;
export declare const enum ConstantStyle {
    allCapsSnake = "snake",
    allCaps = "allCaps",
    camelCase = "camelCase",
    pascalCase = "pascalCase"
}
/**
 * The style to use to name enum members. The name of enum members is important in Java languages as the enum member
 * name is used, e.g. `enum.name()` and `EnumType.valueOf(String)`.
 */
export declare const enum EnumMemberStyle {
    /** Try to preserve the enum value from the specification in the enum entry name */
    preserve = "preserve",
    /** Match the constant style */
    constant = "contant"
}
export interface JavaLikeOptions {
    apiClassPrefix?: string;
    modelClassPrefix?: string;
    modelClassSuffix?: string;
    enumClassPrefix?: string;
    enumClassSuffix?: string;
    nestedModelClassPrefix?: string;
    nestedEnumClassPrefix?: string;
    constantStyle: ConstantStyle;
    enumMemberStyle: EnumMemberStyle;
}
export declare function options(config: CodegenConfig, context: JavaLikeContext): JavaLikeOptions;
export interface JavaLikeContext extends CodegenGeneratorContext {
    reservedWords?: () => string[];
    defaultConstantStyle: ConstantStyle;
    defaultEnumMemberStyle: EnumMemberStyle;
}
export declare function javaLikeGenerator(config: CodegenConfig, context: JavaLikeContext): Pick<CodegenGenerator, 'toClassName' | 'toIdentifier' | 'toEnumMemberName' | 'toConstantName' | 'toSchemaName' | 'toSuggestedSchemaName' | 'toOperationGroupName'>;
